#include "fingering.h"
#include "data/tabtrack.h"
#include "settings.h"

#include <qpainter.h>
#include <qdrawutil.h>
#include <qevent.h>
#include <qbrush.h>
#include <qstring.h>
#include <qscrollbar.h>
#include <QMouseEvent>
#include <QAbstractScrollArea>

Fingering::Fingering(TabTrack *p, QWidget *parent):
    QAbstractScrollArea(parent)
{
	parm = p;

	lastff = 1;

	setFixedSize(parm->string * SCALE + 2 * BORDER + FRETTEXT + SCROLLER,
	             NUMFRETS * SCALE + SCALE + 2 * BORDER + 2 * SPACER + NOTES);
	setFrameStyle(Panel | Sunken);

	verticalScrollBar()->setMinimum(1);
	verticalScrollBar()->setMaximum(parm->frets-NUMFRETS + 1);
	verticalScrollBar()->setSingleStep(5);
	verticalScrollBar()->setValue(1);
	verticalScrollBar()->setGeometry(width() - SCROLLER, 0, SCROLLER, height());
	connect(verticalScrollBar(), SIGNAL(valueChanged(int)), SLOT(setFirstFret(int)));

	clear();
}

void Fingering::clear()
{
	for (int i = 0; i < parm->string; i++)
		appl[i] = -1;

	emit chordChange();
}

void Fingering::setFinger(int string, int fret)
{
	if (appl[string] != fret) {
		appl[string] = fret;

		emit chordChange();
	}
}

void Fingering::setFingering(const int a[MAX_STRINGS])
{
    int f = 24; // GREYFIX
    bool noff = TRUE;

    for (int i = 0; i < parm->string; i++) {
        if ((a[i] < f) && (a[i] > 0))
            f = a[i];
        if (a[i] > 5)
            noff = FALSE;
    }

    if (noff)
        f = 1;

    verticalScrollBar()->setValue(f);

    for (int i = 0;i < MAX_STRINGS; i++)
        appl[i] = a[i];

	viewport()->update();
    emit chordChange();
}

void Fingering::setFirstFret(int fret)
{
    for (int i = 0; i < parm->string; i++)
	if (appl[i] > 0)
	    appl[i] = appl[i] - lastff + fret;

    lastff = fret;

    emit chordChange();
}

void Fingering::mouseHandle(const QPoint &pos, bool domute)
{
    int i = (pos.x() - BORDER - FRETTEXT) / SCALE;
    int j = 0;
    if (pos.y() > BORDER + SCALE + 2 * SPACER)
        j = (pos.y() - BORDER - SCALE - 2 * SPACER) / SCALE + verticalScrollBar()->value();

    if ((domute) && (appl[i] == j))
        j = -1;

    if (!((i < 0) || (i >= parm->string) || (j >= verticalScrollBar()->value() + NUMFRETS)))
        setFinger(i, j);

	viewport()->update();
}

void Fingering::mouseMoveEvent(QMouseEvent *e)
{
	mouseHandle(e->pos(), FALSE);
}

void Fingering::mousePressEvent(QMouseEvent *e)
{
	if (e->button() == Qt::LeftButton)
		mouseHandle(e->pos(), TRUE);
}

void Fingering::paintEvent(QPaintEvent *)
{
	QPainter painter(viewport());
	QPainter* p = &painter;

	int barre, eff;

	p->setRenderHints(QPainter::Antialiasing | QPainter::HighQualityAntialiasing);

	// Horizontal separator line

	p->drawLine(BORDER + FRETTEXT, BORDER + SCALE + SPACER,
	            BORDER + FRETTEXT + parm->string * SCALE, BORDER + SCALE + SPACER);

	// Horizontal lines

	for (int i = 0; i <= NUMFRETS; i++)
		p->drawLine(SCALE / 2 + BORDER + FRETTEXT, BORDER + SCALE + 2 * SPACER + i * SCALE,
		            SCALE / 2 + BORDER + parm->string * SCALE - SCALE + FRETTEXT,
		            BORDER +SCALE + 2 * SPACER + i * SCALE);

	// Beginning fret number

	QString tmp;
	tmp.setNum(verticalScrollBar()->value());
	p->drawText(BORDER - SPACER, BORDER + SCALE + 2 * SPACER, 50, 50, Qt::AlignLeft | Qt::AlignTop, tmp);

	// Vertical lines, fingering and note names

	for (int i = 0; i < parm->string; i++) {
		p->drawLine( i * SCALE + BORDER + SCALE / 2 + FRETTEXT, BORDER + SCALE + 2 * SPACER,
		         i * SCALE + BORDER + SCALE / 2 + FRETTEXT,
		         BORDER + SCALE + 2 * SPACER + NUMFRETS *SCALE);
		if (appl[i] == -1) {
			p->drawLine(i * SCALE + BORDER + CIRCBORD + FRETTEXT, BORDER + CIRCBORD,
			            i * SCALE + BORDER + SCALE - CIRCBORD + FRETTEXT, BORDER + SCALE - CIRCBORD);
			p->drawLine(i * SCALE + BORDER + SCALE - CIRCBORD + FRETTEXT, BORDER +CIRCBORD,
			            i * SCALE + BORDER + CIRCBORD + FRETTEXT, BORDER + SCALE - CIRCBORD);
		} else {
			if (appl[i] == 0) {
				p->setBrush(Qt::NoBrush);
				p->drawEllipse(i * SCALE + BORDER + CIRCBORD + FRETTEXT,
				               BORDER + CIRCBORD, CIRCLE, CIRCLE);
			} else {
				p->setBrush(Qt::SolidPattern);
				p->drawEllipse(i * SCALE + BORDER + CIRCBORD + FRETTEXT,
				               BORDER + SCALE + 2 * SPACER + (appl[i] - verticalScrollBar()->value()) * SCALE + CIRCBORD,
				               CIRCLE, CIRCLE);
			};
			p->drawText(BORDER + FRETTEXT + i * SCALE, BORDER + NUMFRETS * SCALE + SCALE + 2 * SPACER,
			            SCALE, 30, Qt::AlignHCenter | Qt::AlignTop,
			            Settings::noteName((appl[i] + parm->tune[i]) % 12));
		}
	}

	// Analyze & draw barre

	p->setBrush(Qt::SolidPattern);

	for (int i = 0; i < NUMFRETS; i++) {
		barre = 0;
		while ((appl[parm->string - barre - 1] >= (i + verticalScrollBar()->value())) ||
		       (appl[parm->string - barre - 1] == -1)) {
			barre++;
			if (barre>parm->string - 1)
				break;
		}

		while ((appl[parm->string - barre] != (i + verticalScrollBar()->value())) && (barre > 1))
			barre--;

		eff = 0;
		for (int j = parm->string - barre; j < parm->string; j++) {
			if (appl[j] != -1)
				eff++;
		}

		if (eff > 2) {
			p->drawRect((parm->string - barre) * SCALE + SCALE / 2 + BORDER + FRETTEXT,
			            BORDER + SCALE + 2 * SPACER + i * SCALE + CIRCBORD,
			            (barre - 1) * SCALE, CIRCLE);
		}
	}
}
